@@ -42,6 +42,7 @@ Update noise values in SDarn fields when these can be calculated.
 """
 import logging
 import numpy as np
+import deepdish as dd
 
 from datetime import datetime
 from typing import Union
@@ -361,21 +362,43 @@ class BorealisConvert(BorealisRead):
             for record_key, record in self.borealis_records.items():
                 sample_spacing = int(record['tau_spacing'] /
                                      record['tx_pulse_len'])
-                normal_blanked_1 = record['pulses'] * sample_spacing
-                normal_blanked_2 = normal_blanked_1 + 1
-                blanked = np.concatenate((normal_blanked_1, normal_blanked_2))
-                blanked = np.sort(blanked)
+
+                # Check to see if tagged version. If not, use 255.255
+                git_hash = record['borealis_git_hash'].split('-')[0]
+                major_version, minor_version = git_hash.split('.')
+                if major_version[0] == 'v':
+                    borealis_major_revision = major_version[1:]
+                    borealis_minor_revision = minor_version
+                else:
+                    borealis_major_revision = 255
+                    borealis_minor_revision = 255
+
+                if borealis_major_revision == 0 and borealis_minor_revision <= 5:
+                    # Bfiq generated with borealis v0.5 or older
+                    blanked = record['pulses'] * sample_spacing
+                else:
+                    # Bfiq generated with borealis v0.6 or newer, or untagged version
+                    normal_blanked_1 = record['pulses'] * sample_spacing
+                    if np.array_equal(normal_blanked_1, record['blanked_samples']):
+                        # If file generated with untagged borealis version, it could be like v0.5
+                        # and still have valid blanked samples, so we check that
+                        blanked = normal_blanked_1
+                    else:
+                        normal_blanked_2 = normal_blanked_1 + 1
+                        blanked = np.concatenate((normal_blanked_1, normal_blanked_2))
+                        blanked = np.sort(blanked)
+
                 if not np.array_equal(record['blanked_samples'], blanked):
                     raise borealis_exceptions.\
                             BorealisConvert2IqdatError(
                                 'Increased complexity: Borealis bfiq file'
-                                ' record {} blanked_samples {} does not equate'
-                                ' to pulses array converted to sample number '
-                                '{} * {}'.format(record_key,
-                                                 record['blanked_samples'],
-                                                 record['pulses'],
-                                                 int(record['tau_spacing'] /
-                                                     record['tx_pulse_len'])))
+                                ' record {} blanked_samples {} is not correct'
+                                ' for pulses array converted to sample number '
+                                '{} * {}.'.format(record_key,
+                                                  record['blanked_samples'],
+                                                  record['pulses'],
+                                                  int(record['tau_spacing'] /
+                                                      record['tx_pulse_len'])))
                 if not all([x == 0 for x in record['pulse_phase_offset']]):
                     raise borealis_exceptions.\
                             BorealisConvert2IqdatError(
@@ -391,14 +414,11 @@ class BorealisConvert(BorealisRead):
 
         The file is convertible if:
             - the origin filetype is rawacf
-            - the blanked_samples array = pulses array for all records
+            - the blanked_samples array = pulses array for all records with Borealis v0.5 or older;
+            - the blanked_samples array is the same as the pulses array * sample number, but also
+              blanks the sample after each pulse, for Borealis v0.6 or newer
             - the pulse_phase_offset array contains all zeroes for all records
 
-        TODO: should this fail for multiple beams in the same
-        integration time. IE, is it ok for dmap files to have multiple
-        records with same origin time and timestamps due to a different
-        beam azimuth.
-
         Raises
         ------
         BorealisConversionTypesError
@@ -414,24 +434,47 @@ class BorealisConvert(BorealisRead):
                                                  self.borealis_filetype,
                                                  self.__allowed_conversions)
         else:  # There are some specific things to check
+            
             for record_key, record in self.borealis_records.items():
                 sample_spacing = int(record['tau_spacing'] /
                                      record['tx_pulse_len'])
-                normal_blanked_1 = record['pulses'] * sample_spacing
-                normal_blanked_2 = normal_blanked_1 + 1
-                blanked = np.concatenate((normal_blanked_1, normal_blanked_2))
-                blanked = np.sort(blanked)
+
+                # Check to see if tagged version. If not, use 255.255
+                git_hash = record['borealis_git_hash'].split('-')[0]
+                major_version, minor_version = git_hash.split('.')
+                if major_version[0] == 'v':
+                    borealis_major_revision = major_version[1:]
+                    borealis_minor_revision = minor_version
+                else:
+                    borealis_major_revision = 255
+                    borealis_minor_revision = 255
+
+                if borealis_major_revision == 0 and borealis_minor_revision <= 5:
+                    # Rawacf generated with borealis v0.5 or older
+                    blanked = record['pulses'] * sample_spacing
+                else:
+                    # Rawacf generated with borealis v0.6 or newer, or untagged version
+                    normal_blanked_1 = record['pulses'] * sample_spacing
+                    if np.array_equal(normal_blanked_1, record['blanked_samples']):
+                        # If file generated with untagged borealis version, it could be like v0.5
+                        # and still have valid blanked samples, so we check that
+                        blanked = normal_blanked_1
+                    else:
+                        normal_blanked_2 = normal_blanked_1 + 1
+                        blanked = np.concatenate((normal_blanked_1, normal_blanked_2))
+                        blanked = np.sort(blanked)
+
                 if not np.array_equal(record['blanked_samples'], blanked):
                     raise borealis_exceptions.\
                             BorealisConvert2RawacfError(
                                 'Increased complexity: Borealis rawacf file'
-                                ' record {} blanked_samples {} does not equate'
-                                ' to pulses array converted to sample number '
-                                '{} * {}'.format(record_key,
-                                                 record['blanked_samples'],
-                                                 record['pulses'],
-                                                 int(record['tau_spacing'] /
-                                                     record['tx_pulse_len'])))
+                                ' record {} blanked_samples {} is not correct'
+                                ' for pulses array converted to sample number '
+                                '{} * {}.'.format(record_key,
+                                                  record['blanked_samples'],
+                                                  record['pulses'],
+                                                  int(record['tau_spacing'] /
+                                                      record['tx_pulse_len'])))
 
         return True
 
@@ -469,12 +512,12 @@ class BorealisConvert(BorealisRead):
         try:
             recs = []
             for record in self.borealis_records.items():
-                sdarn_record_dict = \
+                record_dict_list = \
                         self.__convert_bfiq_record(self.borealis_slice_id,
                                                    record,
                                                    self.borealis_filename,
                                                    self.scaling_factor)
-                recs.append(sdarn_record_dict)
+                recs.extend(record_dict_list)
             self._sdarn_dict = recs
             self._sdarn_dmap_records = dict2dmap(recs)
         except Exception as e:
@@ -484,7 +527,7 @@ class BorealisConvert(BorealisRead):
     def __convert_bfiq_record(borealis_slice_id: int,
                               borealis_bfiq_record: tuple,
                               origin_string: str,
-                              scaling_factor: int = 1) -> dict:
+                              scaling_factor: int = 1) -> list:
         """
         Converts a single record dict of Borealis bfiq data to a SDARN DMap
         record dict.
@@ -539,6 +582,7 @@ class BorealisConvert(BorealisRead):
         offset = 2 * record_dict['antenna_arrays_order'].shape[0] * \
             record_dict['num_samps']
 
+        record_dict_list = []
         for beam_index, beam in enumerate(record_dict['beam_nums']):
             # grab this beam's data
             # shape is now num_antenna_arrays x num_sequences
@@ -567,6 +611,20 @@ class BorealisConvert(BorealisRead):
 
             int_data = np.array(int_data, dtype=np.int16)
 
+            # AGC Status Word only introduced in Borealis v0.6 onwards,
+            # so it can be set to zero if not present
+            if 'agc_status_word' not in record_dict.keys():
+                agc_sw = 0
+            else:
+                agc_sw = record_dict['agc_status_word']
+
+            # Low Power Status Word only introduced in Borealis v0.6 onwards,
+            # so it can be set to zero if not present
+            if 'lp_status_word' not in record_dict.keys():
+                lp_sw = 0
+            else:
+                lp_sw = record_dict['lp_status_word']
+
             # flattening done in convert_to_dmap_datastructures
             sdarn_record_dict = {
                 'radar.revision.major': np.int8(borealis_major_revision),
@@ -616,8 +674,8 @@ class BorealisConvert(BorealisRead):
                 # smsep is in us; conversion from seconds
                 'smsep': np.int16(1e6 / record_dict['rx_sample_rate']),
                 'ercod': np.int16(0),
-                'stat.agc': np.int16(record_dict['agc_status_word']),
-                'stat.lopwr': np.int16(record_dict['lp_status_word']),
+                'stat.agc': np.int16(agc_sw),
+                'stat.lopwr': np.int16(lp_sw),
                 # TODO: currently not implemented
                 'noise.search': np.float32(record_dict['noise_at_freq'][0]),
                 # TODO: currently not implemented
@@ -692,7 +750,8 @@ class BorealisConvert(BorealisRead):
                                  dtype=np.int32),
                 'data': int_data
             }
-        return sdarn_record_dict
+            record_dict_list.append(sdarn_record_dict)
+        return record_dict_list
 
     def _convert_rawacf_to_rawacf(self):
         """
@@ -717,12 +776,12 @@ class BorealisConvert(BorealisRead):
         try:
             recs = []
             for record in self.borealis_records.items():
-                sdarn_record_dict = \
+                record_dict_list = \
                         self.__convert_rawacf_record(self.borealis_slice_id,
                                                      record,
                                                      self.borealis_filename,
                                                      self.scaling_factor)
-                recs.append(sdarn_record_dict)
+                recs.extend(record_dict_list)
             self._sdarn_dict = recs
             self._sdarn_dmap_records = dict2dmap(recs)
         except Exception as e:
@@ -732,7 +791,7 @@ class BorealisConvert(BorealisRead):
     def __convert_rawacf_record(borealis_slice_id: int,
                                 borealis_rawacf_record: tuple,
                                 origin_string: str,
-                                scaling_factor: int = 1) -> dict:
+                                scaling_factor: int = 1) -> list:
         """
         Converts a single record dict of Borealis rawacf data to a SDARN DMap
         record dict.
@@ -790,23 +849,29 @@ class BorealisConvert(BorealisRead):
             borealis_major_revision = 255
             borealis_minor_revision = 255
 
+        record_dict_list = []
         for beam_index, beam in enumerate(record_dict['beam_nums']):
             # this beam, all ranges lag 0
             lag_zero = shaped_data['main_acfs'][beam_index, :, 0]
-            lag_zero[-10:] = shaped_data['main_acfs'][beam_index, -10:, -1]
+
+            # Historically, the following line was un-commented. It's purpose was to replace the lag0 data for far
+            # ranges which were contaminated by the second pulse in a sequence. However, it only worked for a small
+            # subset of conditions; specifically, it failed for experiments where the second pulse occurred earlier
+            # or the number of range gates was larger than normal. This replacement is now handled in borealis, and
+            # is more flexible to deal with the variety of different conditions possible from experiments.
+            # lag_zero[-10:] = shaped_data['main_acfs'][beam_index, -10:, -1]
+
             lag_zero_power = (lag_zero.real**2 + lag_zero.imag**2)**0.5
 
             correlation_dict = {}
             for key in shaped_data:
                 # num_ranges x num_lags (complex)
                 this_correlation = shaped_data[key][beam_index, :, :-1]
-                # set the lag0 to the alternate lag0 for the end of the
-                # array (when interference of first pulse would occur)
-                this_correlation[-10:, 0] = \
-                    shaped_data[key][beam_index, -10:, -1]
-                # shape num_beams x num_ranges x num_la gs, now
-                # num_ranges x num_lags-1 b/c alternate lag-0 combined
-                # with lag-0 (only used for last ranges)
+
+                ##### Similar to above, this line has been commented out to avoid far-range lag0 replacement
+                ##### as it is now handled in borealis.
+                # this_correlation[-10:, 0] = \
+                #     shaped_data[key][beam_index, -10:, -1]
 
                 # (num_ranges x num_lags, flattened)
                 flattened_data = np.array(this_correlation).flatten()
@@ -826,6 +891,20 @@ class BorealisConvert(BorealisRead):
                 # place the SDARN-style array in the dict
                 correlation_dict[key] = new_data
 
+            # AGC Status Word only introduced in Borealis v0.6 onwards,
+            # so it can be set to zero if not present
+            if 'agc_status_word' not in record_dict.keys():
+                agc_sw = 0
+            else:
+                agc_sw = record_dict['agc_status_word']
+
+            # Low Power Status Word only introduced in Borealis v0.6 onwards,
+            # so it can be set to zero if not present
+            if 'lp_status_word' not in record_dict.keys():
+                lp_sw = 0
+            else:
+                lp_sw = record_dict['lp_status_word']
+
             sdarn_record_dict = {
                 'radar.revision.major': np.int8(borealis_major_revision),
                 'radar.revision.minor': np.int8(borealis_minor_revision),
@@ -874,8 +953,8 @@ class BorealisConvert(BorealisRead):
                 'lagfr': np.int16(record_dict['first_range_rtt']),
                 'smsep': np.int16(1e6/record_dict['rx_sample_rate']),
                 'ercod': np.int16(0),
-                'stat.agc': np.int16(record_dict['agc_status_word']),
-                'stat.lopwr': np.int16(record_dict['lp_status_word']),
+                'stat.agc': np.int16(agc_sw),
+                'stat.lopwr': np.int16(lp_sw),
                 # TODO: currently not implemented
                 'noise.search': np.float32(record_dict['noise_at_freq'][0]),
                 # TODO: currently not implemented
@@ -924,5 +1003,6 @@ class BorealisConvert(BorealisRead):
                 'acfd': correlation_dict['main_acfs'],
                 'xcfd': correlation_dict['xcfs']
             }
+            record_dict_list.append(sdarn_record_dict)
 
-        return sdarn_record_dict
+        return record_dict_list
